"""SCEX Client for making requests to the SCE Server"""

import logging

from .api_handler import ApiHandler
from .constants import (
    BALLOON_PAYMENT_FINAL_INPUT_XML,
    BALLOON_PAYMENT_REGULAR_INPUT_XML,
    EQUAL_PAYMENT_INPUT_XML,
    AccuralCode,
    PaymentsPerYear,
)
from .utils import ValidationError, validate_inputs

LOGGER = logging.getLogger("root")


class ScexClient:
    def __init__(self, host: str, api_key: str):
        self.__request_url = f"{host}/scex/api/v1/request"
        self._api_handler = ApiHandler(self.__request_url, api_key)

    @property
    def url(self):
        return self.__request_url

    def __initialize_defaults(self, options: dict):
        LOGGER.info("Setting default values")

        options.setdefault("Account", 1)
        options.setdefault("DataDirPath", "/etc/sce/*")
        options.setdefault("AccuralCode", AccuralCode.ACTUAL_360_SIMPLE.value)
        options.setdefault("AprType", "default")
        options.setdefault("UseMAPR", "false")
        options.setdefault("HideAmort", "false")
        options.setdefault("Lastday", "false")
        options.setdefault("OddFinal", "true")
        options.setdefault("ValidateInput", "true")
        options.setdefault("ValidateOutput", "true")
        options.setdefault("TotalDown", 0)

        if "PPY" not in options:
            options["PPY"] = PaymentsPerYear.MONTHLY.value[0]
        else:
            try:
                PaymentsPerYear(options["PPY"])
            except ValueError:
                raise ValidationError("Invalid payment schedule")

        options["TilaRespa"] = "<TILARESPA2015/>" if "TilaRespa" in options else ""

        LOGGER.info("Validating equal payment loan input")
        validate_inputs(options)

        return options

    # def load_logger(self, logging_config=LOGGING_CONFIG):
    #     logging.config.dictConfig(LOGGING_CONFIG)
    #     LOGGER = logging.getLogger("scex_py")

    def process_equal_payment_loan(self, options: dict, log_config: dict = None):
        options = self.__initialize_defaults(options)
        LOGGER.info("Loading raw XML for Equal Payment Loan")

        with open(EQUAL_PAYMENT_INPUT_XML) as equal_payment_input_file:
            equal_payment_input = equal_payment_input_file.read()

        equal_payment_input = equal_payment_input.format(**options)

        LOGGER.info("Calculating Equal Payment loan")
        return self._api_handler.send_request("POST", payload=equal_payment_input, log_config=log_config)

    def process_balloon_payment_loan_regular(self, options: dict, log_config: dict = None):
        options = self.__initialize_defaults(options)

        if "RegPmt" not in options:
            raise ValidationError("Regular Payment value is required")

        LOGGER.info("Loading raw XML for Balloon Payment Loan with Regular Payment")

        with open(BALLOON_PAYMENT_REGULAR_INPUT_XML) as balloon_regular_payment_input_file:
            balloon_regular_payment_input = balloon_regular_payment_input_file.read()

        balloon_regular_payment_input = balloon_regular_payment_input.format(**options)

        LOGGER.info("Calculating Balloon Payment loan with Regular Payment")
        return self._api_handler.send_request("POST", payload=balloon_regular_payment_input, log_config=log_config)

    def process_balloon_payment_loan_final(self, options: dict, log_config: dict = None):
        options = self.__initialize_defaults(options)

        if "FinalPmt" not in options:
            raise ValidationError("Final Payment value is required")

        LOGGER.info("Loading raw XML for Balloon Payment Loan with Final Payment")

        with open(BALLOON_PAYMENT_FINAL_INPUT_XML) as balloon_final_payment_input_file:
            balloon_final_payment_input = balloon_final_payment_input_file.read()

        balloon_final_payment_input = balloon_final_payment_input.format(**options)

        LOGGER.info("Calculating Balloon Payment loan with Final Payment")
        return self._api_handler.send_request("POST", payload=balloon_final_payment_input, log_config=log_config)



import logging

import requests

from .constants import API_KEY_HEADER

LOGGER = logging.getLogger("root")


class ApiHandler:
    """Base class for calling any external APIs"""

    REQUEST_TIMEOUT = 30

    def __init__(self, url: str, api_key: str, logging: bool = True):
        self._url = url
        self.__api_key = api_key
        self.__logging = logging

    @property
    def url(self):
        return self._url

    def get_headers(self, **kwargs):
        """Builds and return headers for each request"""
        kwargs.setdefault("Content-Type", "application/xml")
        kwargs.setdefault(API_KEY_HEADER, self.__api_key)
        return kwargs

    def send_request(
        self,
        method,
        params=None,
        payload=None,
        log_config: dict = None,
        headers: dict = None,
    ):
        """Send API request for the given URL with the specified method, params and payload"""
        if not headers:
            headers = self.get_headers()

        if self.__logging:
            LOGGER.info("[SCEX] - [%s: %s]", method, self.url)

        log_entry = None

        if log_config:
            if (
                "model" not in log_config
                or "user" not in log_config
                or "loan" not in log_config
                or "timezone" not in log_config
            ):
                raise Exception("Invalid log dict")

        try:
            if log_config:
                log_entry = log_config["model"](
                    loan=log_config["loan"],
                    requested_by=log_config["user"],
                    request_url=f"{method}: {self.url}",
                    request_headers=headers,
                    request_body=payload,
                    request_time=log_config["timezone"].now(),
                )

            response = requests.request(
                method,
                self.url,
                timeout=self.REQUEST_TIMEOUT,
                headers=headers,
                params=params,
                data=payload,
            )

            if self.__logging:
                LOGGER.info(
                    "Received [%s] response for [%s: %s]",
                    response.status_code,
                    method,
                    self.url,
                )

            if log_entry:
                log_entry.response_code = response.status_code
                log_entry.response_body = response.text
                log_entry.response_time = log_entry.request_time + response.elapsed
                log_entry.save()

            response.raise_for_status()
            return response
        except requests.HTTPError as excp:
            if self.__logging:
                LOGGER.error(
                    "SCEX API Failed. Received [%s] response for [%s: %s]",
                    response.status_code,
                    method,
                    self.url,
                )

            raise Exception(f"Failed to get success response from SCEX. Response: [{response.text}]") from excp



"""Module for storing constants for the SCEX Client package"""

from enum import Enum
from pathlib import Path

API_KEY_HEADER = "x-sce-apikey"

BASE_DIR = Path(__file__).resolve().parent
EQUAL_PAYMENT_INPUT_XML = BASE_DIR.joinpath("input_xml", "equal_payment_loan.xml")
BALLOON_PAYMENT_REGULAR_INPUT_XML = BASE_DIR.joinpath("input_xml", "balloon_payment", "regular.xml")
BALLOON_PAYMENT_FINAL_INPUT_XML = BASE_DIR.joinpath("input_xml", "balloon_payment", "final.xml")

LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": True,
    "formatters": {
        "development": {
            "format": (
                "%(process)d  %(thread)d  %(asctime)-25s %(module)-15s %(funcName)-30s %(levelname)-10s -  %(message)s"
            ),
        },
    },
    "handlers": {
        "development": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "development",
        },
    },
    "loggers": {
        "scex_py": {
            "handlers": ["development"],
            "level": "DEBUG",
            "propagate": False,
        },
    },
}


class AccuralCode(Enum):
    ACTUAL_360_SIMPLE = 210
    ACTUAL_365_SIMPLE = 220
    ACTUAL_360_US_RULE = 310
    ACTUAL_365_US_RULE = 320
    UNIT_PERIOD_SIMPLE_WITH_360_DAY_DIVISOR = 201
    UNIT_PERIOD_SIMPLE_WITH_365_DAY_DIVISOR = 202
    UNIT_PERIOD_SIMPLE_WITH_360_CALENDAR_360_DAY_DIVISOR = 204
    UNIT_PERIOD_SIMPLE_WITH_360_CALENDAR_365_DAY_DIVISOR = 205


class PaymentsPerYear(Enum):
    ANNUAL = "annual"
    SEMI_ANNUAL = "semiannual"
    QUATERLY = "quarterly"
    BI_MONTHLY = "bimonthly"
    MONTHLY = "monthly"
    SEMI_MONTHLY = "semimonthly"
    BI_WEEKLY = "biweekly"
    WEEKLY = "weekly"



"""Module for defining utilities"""

from datetime import datetime

from .constants import AccuralCode


class ValidationError(Exception):
    pass


def validate_date_input(options: dict, key: str):
    try:
        datetime.strptime(options[key], "%Y-%m-%d")
    except KeyError:
        raise ValidationError(f"{key} is required")
    except ValueError:
        raise ValidationError("Date should be in `%Y-%m-%d` format")


def validate_inputs(options: dict):
    validate_date_input(options, "LoanDate")
    validate_date_input(options, "PmtDate")
    validate_date_input(options, "IntStartDate")

    if "Proceeds" not in options:
        raise ValidationError("Proceeds is required")

    if "Term" not in options:
        raise ValidationError("Term is required")

    if "IntRate" not in options:
        raise ValidationError("IntRate is required")

    try:
        AccuralCode(options["AccuralCode"])
    except ValueError:
        raise ValidationError("Accural code not supported")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scex_py']

package_data = \
{'': ['*'], 'scex_py': ['input_xml/*', 'input_xml/balloon_payment/*']}

install_requires = \
['requests>=2.26.0,<2.27.0', 'xmltodict>=0.12.0,<0.13.0']

setup_kwargs = {
    'name': 'scex-py',
    'version': '1.0.3',
    'description': '',
    'long_description': None,
    'author': 'Jatin Goel',
    'author_email': 'jatin.goel@thesummitgrp.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
